{$IFDEF ASCII_SDL}
unit ascii_sdl;

interface

uses
	sysutils,
{$IFDEF SDL2}
	SDL2,SDL2_TTF
{$ELSE SDL2}
	SDL,SDL_TTF
{$ENDIF SDL2}
	;

Type
	vgfx_rect = Record
		X,Y,W,H: Byte;
	end;

const
	VG_FGColor: TSDL_Color		= ( r:192; g:192; b:192 );	{ LightGray }
	VG_BGColor: TSDL_Color		= ( r:  0; g:  0; b:  0 );	{ Black }
	VG_X: Byte = 1;	{ Cursor Position. }
	VG_Y: Byte = 1;	{ Cursor Position. }
	VG_Window: vgfx_rect = ( X:1 ; Y:1 ; W:80 ; H:25 );


{$IFDEF PATCH_AUDIO}
Procedure PlaySoundEffect_Direct_Raw( const Name: String );
{$ENDIF PATCH_AUDIO}

Procedure DoFlip;

Procedure Window( const X1,Y1,X2,Y2: LongInt );
Procedure MaxClipZone;
Procedure ClipZone( const ZoneNumber: Integer );
Procedure ClrZone_Raw( const ZoneNumber: Integer );
Procedure ClrScreen_Raw;
Procedure ClrScr;
Procedure ClrEOL;
Procedure CursorOn;
Procedure CursorOff;
Function WhereX: Integer;
Function WhereY: Integer;
Procedure GotoXY( const X,Y: Integer );
Procedure TextColor( const C: TSDL_Color );
Procedure TextBackground( const C: TSDL_Color );
{$IFDEF PATCH_HEART}
Function New_Conv_ToUni16( const arg_msg: String ): PUInt16;
{$ENDIF PATCH_HEART}
Procedure Write( const S: String );
Procedure WriteLn;
Procedure WriteLn( const S: String );
Procedure Delay( const MS: Word );
{$IFDEF PATCH_I18N}
  {$IFDEF PATCH_HEART}
    {$IFDEF SDL2}
function RPGKey_Raw( const nonblocking: Boolean; var Unicode: String; const GetStringMode: Boolean ): Char;
    {$ELSE SDL2}
function RPGKey_Raw( const nonblocking: Boolean; var Unicode: Word; const GetStringMode: Boolean ): Char;
    {$ENDIF SDL2}
  {$ELSE PATCH_HEART}
    {$IFDEF SDL2}
function RPGKey_Raw( var Unicode: String; const GetStringMode: Boolean ): Char;
    {$ELSE SDL2}
function RPGKey_Raw( var Unicode: Word; const GetStringMode: Boolean ): Char;
    {$ENDIF SDL2}
  {$ENDIF PATCH_HEART}
  {$IFDEF SDL2}
function RPGKey_Raw( var Unicode: String ): Char;
  {$ELSE SDL2}
function RPGKey_Raw( var Unicode: Word ): Char;
  {$ENDIF SDL2}
{$ENDIF PATCH_I18N}
Function RPGKey_Raw: Char;
{$IFDEF PATCH_GH}
Function GetStringFromUser_Raw( const Prompt: String; const Init_text: String; var canceled: Boolean ): String;
Function GetStringFromUser_Raw( const Prompt: String; const Init_text: String ): String;
{$ENDIF PATCH_GH}
Function GetStringFromUser_Raw( const Prompt: String ): String;


implementation

uses
{$IFDEF PATCH_GH}
	errmsg,pseudosmartpointer,
{$ELSE PATCH_GH}
  {$IFDEF DEBUG}
	errmsg,
  {$ENDIF DEBUG}
{$ENDIF PATCH_GH}
{$IFDEF WITH_TENC}
	termenc,
{$ENDIF WITH_TENC}
	gears_base,
	gears,
	texutil,
	ui4gh,
	congfx,context
{$IFDEF PATCH_AUDIO}
  {$IFDEF SDL2}
	,SDL2_Mixer
  {$ELSE SDL2}
	,SDL_Mixer
  {$ENDIF SDL2}
{$ENDIF PATCH_AUDIO}
{$IF DEFINED(WINDOWS) and DEFINED(WITHOUT_SDLIM)}
	,w32eb
{$ENDIF}
	,ascii_sdlinit
{$IFDEF PATCH_HEART}
	,qrl_viewer
{$ENDIF PATCH_HEART}
{$IF DEFINED(PATCH_GH) and DEFINED(SDL2) and DEFINED(BSD)}
	,libfenv
{$ENDIF}
	;

var
	RPGKey_LastEvent: String = '';
{$IFDEF PATCH_HEART}
	Last_Clock_Update: UInt32;
{$ENDIF PATCH_HEART}


{$IFDEF PATCH_AUDIO}
Procedure PlaySoundEffect_Direct_Raw( const Name: String );
var
	S: SensibleSoundEffectListPtr;
begin
	if ( Length(Name) < 3 ) then exit;

	S := SoundEffectList;
	while ( NIL <> S ) do begin
		if ( Name = S^.Name ) then begin
			Mix_PlayChannel( -1 , S^.Chunk , 0 );
			exit;
		end;
		S := S^.Next;
	end;
	ErrorMessage_fork( 'A sound effect file "' + Name + '" was not found.' );
end;
{$ENDIF PATCH_AUDIO}


Procedure DoFlip;
	{ Update the screen. }
begin
{$IFDEF SDL2}
	SDL_UpdateWindowSurface( Game_Window );
{$ELSE SDL2}
	SDL_Flip( Game_Screen );
{$ENDIF SDL2}
end;


Procedure Window( const X1,Y1,X2,Y2: LongInt );
begin
	VG_Window.X := X1;
	VG_Window.Y := Y1;
	VG_Window.W := ( X2 - X1 + 1 );
	VG_Window.H := ( Y2 - Y1 + 1 );
	VG_X := VG_Window.X;
	VG_Y := VG_Window.Y;
end;

Procedure MaxClipZone;
	{ Restore the clip area to the maximum possible area. }
begin
	VG_Window.X := 1;
	VG_Window.Y := 1;
	VG_Window.W := ScreenColumns;
	VG_Window.H := ScreenRows;
	VG_X := VG_Window.X;
	VG_Y := VG_Window.Y;
end;

Procedure ClipZone( const ZoneNumber: Integer );
	{ Set the clipping bounds to this defined zone. }
begin
	ascii_sdl.Window( ScreenZone[ZoneNumber,1] , ScreenZone[ZoneNumber,2] , ScreenZone[ZoneNumber,3] , ScreenZone[ZoneNumber,4] );
end;

Procedure ClrZone_Raw( const ZoneNumber: Integer );
begin
	ascii_sdl.Window( ScreenZone[ZoneNumber,1] , ScreenZone[ZoneNumber,2] , ScreenZone[ZoneNumber,3] , ScreenZone[ZoneNumber,4] );
	ascii_sdl.TextBackground( Black );
	ascii_sdl.ClrScr;
	ascii_sdl.MaxClipZone;
end;

Procedure ClrZone( const Z: vgfx_rect );
var
	SZ: TSDL_Rect;
begin
	SZ.X := (LongInt(Z.X) - 1) * CharWidth;
	SZ.Y := (LongInt(Z.Y) - 1) * CharHeight;
	SZ.W := LongInt(Z.W) * CharWidth;
	SZ.H := LongInt(Z.H) * CharHeight;
	SDL_FillRect( Game_Screen , @SZ , SDL_MapRGB( Game_Screen^.Format , 0 , 0 , 0 ) );
end;

Procedure ClrScreen_Raw;
	{ Clear the entire screen. }
begin
	ascii_sdl.MaxClipZone;
	SDL_FillRect( Game_Screen , NIL , SDL_MapRGBA( Game_Screen^.Format , 0 , 0 , 0 , 0 ) );
	VG_X := VG_Window.X;
	VG_Y := VG_Window.Y;
end;


Procedure ClrScr;
var
	SZ: TSDL_Rect;
begin
	SZ.X := (LongInt(VG_Window.X) - 1) * CharWidth;
	SZ.Y := (LongInt(VG_Window.Y) - 1) * CharHeight;
	SZ.W := LongInt(VG_Window.W) * CharWidth;
	SZ.H := LongInt(VG_Window.H) * CharHeight;
	SDL_FillRect( Game_Screen , @SZ , SDL_MapRGB( Game_Screen^.Format , 0 , 0 , 0 ) );
	VG_X := VG_Window.X;
	VG_Y := VG_Window.Y;
end;

Procedure ClrEOL;
var
	Z: vgfx_rect;
begin
	Z.X := VG_X;
	Z.Y := VG_Y;
	Z.H := 1;
	Z.W := screenwidth - VG_X;
	if Z.W > ( VG_Window.X + VG_Window.W - 1 ) then begin
		Z.W := ( VG_Window.X + VG_Window.W - VG_X );
	end;
	ClrZone( Z );
end;

Procedure CursorOn;
begin
end;

Procedure CursorOff;
begin
end;

Function WhereX: Integer;
begin
	WhereX := ( VG_X - VG_Window.X + 1 );
end;

Function WhereY: Integer;
begin
	WhereY := ( VG_Y - VG_Window.Y + 1 );
end;

Procedure GotoXY( const X,Y: Integer );
	{ Set the write position to the requested coordinates. }
begin
	VG_X := VG_Window.X + X - 1;
	VG_Y := VG_Window.Y + Y - 1;
end;

Procedure TextColor( const C: TSDL_Color );
	{ Set the foreground color. }
begin
	VG_FGColor := C;
end;

Procedure TextBackground( const C: TSDL_Color );
	{ Set the background color. }
begin
	VG_BGColor := C;
end;


Function InWindow( const X , Y: Byte ): Boolean;
	{ Return TRUE if X,Y is in the window, or FALSE otherwise. }
begin
	InWindow := ( X >= VG_Window.X ) and ( Y >= VG_Window.Y ) and ( X < ( VG_Window.X + VG_Window.W ) ) and ( Y < ( VG_Window.Y + VG_Window.H ) );
end;

Function New_Conv_ToUni16( const arg_msg: String ): PUInt16;
const
	WCLen = 576; { 512; }
var
	pmsg: PChar;
	pdst: PWord;
begin
	pmsg := QuickPCopy(arg_msg);
	pdst := PWord(sysutils.StrAlloc(WCLen));
	Conv_ToUni16( pmsg, Length(arg_msg), pdst, WCLen );
	New_Conv_ToUni16 := pdst;
	StrDispose( pmsg );
end;

Function I18N_TTF_RenderText( var font: PTTF_Font; const pline: String; var fg: TSDL_Color ): PSDL_Surface;
var
	pWC: PUInt16;
	bg: TSDL_Color;
begin
	if TERMINAL_bidiRTL then begin
		pWC := New_Conv_ToUni16( Conv_bidiRTL(pline) );
	end else begin
		pWC := New_Conv_ToUni16( pline );
	end;
	if not(SDL_AAFont) and not(SDL_AAFont_Shaded) then begin
		I18N_TTF_RenderText := TTF_RenderUnicode_Solid( font, pWC, fg );
	end else if SDL_AAFont_Shaded then begin
		bg.r := 0;
		bg.b := 0;
		bg.g := 0;
		I18N_TTF_RenderText := TTF_RenderUnicode_Shaded( font, pWC, fg, bg );
	end else begin
		I18N_TTF_RenderText := TTF_RenderUnicode_Blended( font, pWC, fg );
	end;
	StrDispose( PChar( pWC ) );
end;

Procedure Write( const S: String );
var
	MyText: PSDL_Surface;
	MyDest: TSDL_Rect;
begin
	if InWindow( VG_X , VG_Y )  then begin
		MyText := I18N_TTF_RenderText( Game_Font , S , VG_FGColor );
		if ( NIL <> MyText ) then begin
			{ SDL_SetColorKey( MyText , SDL_SRCCOLORKEY , SDL_MapRGB( Game_Screen^.Format , VG_BGColor.r , VG_BGColor.g , VG_BGColor.b ) ); }
		end;

		MyDest.X := (LongInt(VG_X) - 1) * CharWidth;
		MyDest.Y := (LongInt(VG_Y) - 1) * CharHeight;
		MyDest.W := WidthMBCharStr( S ) * CharWidth;
		MyDest.H := CharHeight;
		SDL_FillRect( Game_Screen , @MyDest , SDL_MapRGB( Game_Screen^.Format , VG_BGColor.r , VG_BGColor.g , VG_BGColor.b ) );
		SDL_BlitSurface( MyText , NIL , Game_Screen , @MyDest );

		SDL_FreeSurface( MyText );
	end;
	VG_X := VG_X + WidthMBCharStr( S );
end;

Procedure WriteLn;
begin
	VG_X := VG_Window.X;
	Inc( VG_Y );
end;

Procedure WriteLn( const S: String );
begin
	ascii_sdl.Write( S );
	VG_X := VG_Window.X;
	Inc( VG_Y );
end;


Procedure Delay( const MS: Word );
begin
	SDL_Delay( MS );
end;


{$IFDEF PATCH_I18N}
  {$IFDEF SDL2}
function RPGKey_Raw( var Unicode: String ): Char;
  {$ELSE SDL2}
function RPGKey_Raw( var Unicode: Word ): Char;
  {$ENDIF SDL2}
begin
  {$IFDEF PATCH_HEART}
	RPGKey_Raw := RPGKey_Raw( False , Unicode, True );
  {$ELSE PATCH_HEART}
	RPGKey_Raw := RPGKey_Raw( Unicode, True );
  {$ENDIF PATCH_HEART}
end;

function RPGKey_Raw: Char;
var
    {$IFDEF SDL2}
	dummy: String;
    {$ELSE SDL2}
	dummy: Word;
    {$ENDIF SDL2}
begin
  {$IFDEF PATCH_HEART}
	RPGKey_Raw := RPGKey_Raw( False , dummy, False );
  {$ELSE PATCH_HEART}
	RPGKey_Raw := RPGKey_Raw( dummy, False );
  {$ENDIF PATCH_HEART}
end;
{$ENDIF PATCH_I18N}

{$IFDEF PATCH_I18N}
  {$IFDEF PATCH_HEART}
    {$IFDEF SDL2}
function RPGKey_Raw( const nonblocking: Boolean; var Unicode: String; const GetStringMode: Boolean ): Char;
    {$ELSE SDL2}
function RPGKey_Raw( const nonblocking: Boolean; var Unicode: Word; const GetStringMode: Boolean ): Char;
    {$ENDIF SDL2}
  {$ELSE PATCH_HEART}
    {$IFDEF SDL2}
function RPGKey_Raw( var Unicode: String; const GetStringMode: Boolean ): Char;
    {$ELSE SDL2}
function RPGKey_Raw( var Unicode: Word; const GetStringMode: Boolean ): Char;
    {$ENDIF SDL2}
  {$ENDIF PATCH_HEART}
{$ELSE PATCH_I18N}
function RPGKey_Raw: Char;
{$ENDIF PATCH_I18N}
	{ Read a readable key from the keyboard and return its ASCII value. }
var
	a: String;
	event : TSDL_Event;
	m2: PChar;
	width,height: Integer;
	pmsg: PChar;
{$IFDEF PATCH_GH}
	t: LongInt;
	MouseMoved: Boolean = False;
	i: Integer;
{$ENDIF PATCH_GH}
{$IFDEF SDL2}
  {$IFDEF PATCH_HEART}
	EventWaitResult: Integer;
  {$ENDIF PATCH_HEART}
{$ENDIF SDL2}
begin
	DoFlip;

	a := '';
{$IFDEF PATCH_I18N}
  {$IFDEF SDL2}
	Unicode := '';
  {$ELSE SDL2}
	Unicode := 0;
  {$ENDIF SDL2}
{$ENDIF PATCH_I18N}
{$IFDEF PATCH_HEART}
	if nonblocking then begin
		Last_Clock_Update := SDL_GetTicks;
	end;
{$ENDIF PATCH_HEART}
	repeat
		{ Wait for events. }
{$IFDEF SDL2}
  {$IFDEF PATCH_HEART}
		t := 0;
		if nonblocking then begin
			if SDL_GetTicks < ( Last_Clock_Update + KeyWait - SleepGranularity ) then begin
				t := Last_Clock_Update + KeyWait - SDL_GetTicks;
				if 0 < t then begin
					if t > KeysamplingInterval then t := KeysamplingInterval;
				end;
			end else begin
				MouseMoved := True;
				a := RPK_TimeEvent;
			end;
		end;
		if ( 0 < t ) then begin
			EventWaitResult := SDL_WaitEventTimeout( @event , t );
		end else begin
			EventWaitResult := SDL_PollEvent( @event );
		end;
		if ( ( 1 = EventWaitResult ) and ( event.window.windowID = QRLViewer_WindowID ) ) then begin
			QRL_Viewer_EventDriven( event );
		end else if ( ( 1 = EventWaitResult ) and ( event.window.windowID = Game_WindowID ) ) then begin
  {$ELSE PATCH_HEART}
		if SDL_PollEvent( @event ) = 1 then begin
  {$ENDIF PATCH_HEART}
{$ELSE SDL2}
		if SDL_PollEvent( @event ) = 1 then begin
{$ENDIF SDL2}
			{ See if this event is a keyboard one... }
{$IFDEF SDL2}
			if ( SDL_TEXTINPUT = event.type_ ) then begin
				a := event.text.text;
				if ( #$00 < a[1] ) and ( a[1] < #$80 ) then begin
				end else if GetStringMode then begin
					Unicode := event.text.text;
					a := #$80;
				end;
			end else if event.type_ = SDL_KEYDOWN then begin
				{ Check to see if it was an ASCII character we received. }
  {$IF DEFINED(PATCH_GH) and DEFINED(PATCH_I18N)}
				case event.key.keysym.sym of
					SDLK_Backspace:	a := KBD_BS;
					SDLK_Tab:	a := KBD_TAB;
					SDLK_Return:	a := KBD_RET;
					SDLK_Escape:	a := KBD_ESC;
					SDLK_Delete:	a := KBD_Delete;
					SDLK_KP_0:		if not GetStringMode then a := KBD_Insert;
					SDLK_KP_1:		if not GetStringMode then a := KBD_End;
					SDLK_KP_2:		if not GetStringMode then a := KBD_Down;
					SDLK_KP_3:		if not GetStringMode then a := KBD_PgDn;
					SDLK_KP_4:		if not GetStringMode then a := KBD_Left;
					SDLK_KP_5:		if not GetStringMode then a := KBD_KP5;
					SDLK_KP_6:		if not GetStringMode then a := KBD_Right;
					SDLK_KP_7:		if not GetStringMode then a := KBD_Home;
					SDLK_KP_8:		if not GetStringMode then a := KBD_Up;
					SDLK_KP_9:		if not GetStringMode then a := KBD_PgUp;
					SDLK_KP_Period:		if not GetStringMode then a := KBD_Delete;
					SDLK_KP_Divide:		if not GetStringMode then a := '/';
					SDLK_KP_Multiply:	if not GetStringMode then a := '*';
					SDLK_KP_Minus:		if not GetStringMode then a := '-';
					SDLK_KP_Plus:		if not GetStringMode then a := '+';
					SDLK_KP_Enter:		a := KBD_RET;
					SDLK_KP_Equals:		if not GetStringMode then a := '=';
					SDLK_Up:	a := KBD_Up;
					SDLK_Down:	a := KBD_Down;
					SDLK_Right:	a := KBD_Right;
					SDLK_Left:	a := KBD_Left;
					SDLK_Insert:	a := KBD_Insert;
					SDLK_Home:	a := KBD_Home;
					SDLK_End:	a := KBD_End;
					SDLK_PageUp:	a := KBD_PgUp;
					SDLK_PageDown:	a := KBD_PgDn;
					SDLK_F1:	a := KBD_F1;
					SDLK_F2:	a := KBD_F2;
					SDLK_F3:	a := KBD_F3;
					SDLK_F4:	a := KBD_F4;
					SDLK_F5:	a := KBD_F5;
					SDLK_F6:	a := KBD_F6;
					SDLK_F7:	a := KBD_F7;
					SDLK_F8:	a := KBD_F8;
					SDLK_F9:	a := KBD_F9;
					SDLK_F10:	a := KBD_F10;
					SDLK_F11:	a := KBD_F11;
					SDLK_F12:	a := KBD_F12;
					SDLK_F13:	a := KBD_F13;
					SDLK_F14:	a := KBD_F14;
					SDLK_F15:	a := KBD_F15;
				else
				end;
  {$ELSE}
				case event.key.keysym.sym of
				SDLK_F1:	begin
							pmsg := QuickPCopy( Config_Directory + 'Demo.bmp' );
							SDL_SaveBmp( Game_Screen , pmsg );
							StrDispose( pmsg );
						end;
    {$IFDEF PATCH_JPSSDL}
					SDLK_F9:		if SkipAnim then SkipAnim := False else SkipAnim := True;
    {$ENDIF PATCH_JPSSDL}
    {$IFDEF PATCH_CHEAT}
					SDLK_F12:		if Cheat_Display then if Cheat_Display_SW then Cheat_Display_SW := False else Cheat_Display_SW := True;
    {$ENDIF PATCH_CHEAT}
					SDLK_Up,SDLK_KP8:	a := RPK_Up;
					SDLK_Down,SDLK_KP2:	a := RPK_Down;
					SDLK_Left,SDLK_KP4:	a := RPK_Left;
					SDLK_Right,SDLK_KP6:	a := RPK_Right;
					SDLK_KP7:		a := RPK_UpLeft;
					SDLK_KP9:		a := RPK_UpRight;
					SDLK_KP1:		a := RPK_DownLeft;
					SDLK_KP3:		a := RPK_DownRight;
					SDLK_Backspace:		a := #8;
					SDLK_KP_Enter:		a := #10;
					SDLK_KP5:		a := '5';
				else
				end;
  {$ENDIF}
{$ELSE SDL2}
			if event.type_ = SDL_KEYDOWN then begin
				{ Check to see if it was an ASCII character we received. }
  {$IF DEFINED(PATCH_GH) and DEFINED(PATCH_I18N)}
				if GetStringMode then begin
					if ( 0 < event.key.keysym.unicode ) and ( event.key.keysym.unicode < $80 ) then begin
						a := Char( event.key.keysym.unicode );
					end else begin
						Unicode := event.key.keysym.unicode;
						a := #$80;
					end;
				end else begin
					case event.key.keysym.sym of
						SDLK_Backspace:	a := KBD_BS;
						SDLK_Tab:	a := KBD_TAB;
						SDLK_Return:	a := KBD_RET;
						SDLK_Escape:	a := KBD_ESC;
						SDLK_Delete:	a := KBD_Delete;
						SDLK_KP0:		a := KBD_Insert;
						SDLK_KP1:		a := KBD_End;
						SDLK_KP2:		a := KBD_Down;
						SDLK_KP3:		a := KBD_PgDn;
						SDLK_KP4:		a := KBD_Left;
						SDLK_KP5:		a := KBD_KP5;
						SDLK_KP6:		a := KBD_Right;
						SDLK_KP7:		a := KBD_Home;
						SDLK_KP8:		a := KBD_Up;
						SDLK_KP9:		a := KBD_PgUp;
						SDLK_KP_Period:		a := KBD_Delete;
						SDLK_KP_Divide:		a := '/';
						SDLK_KP_Multiply:	a := '*';
						SDLK_KP_Minus:		a := '-';
						SDLK_KP_Plus:		a := '+';
						SDLK_KP_Enter:		a := KBD_RET;
						SDLK_KP_Equals:		a := '=';
						SDLK_Up:	a := KBD_Up;
						SDLK_Down:	a := KBD_Down;
						SDLK_Right:	a := KBD_Right;
						SDLK_Left:	a := KBD_Left;
						SDLK_Insert:	a := KBD_Insert;
						SDLK_Home:	a := KBD_Home;
						SDLK_End:	a := KBD_End;
						SDLK_PageUp:	a := KBD_PgUp;
						SDLK_PageDown:	a := KBD_PgDn;
						SDLK_F1:	a := KBD_F1;
						SDLK_F2:	a := KBD_F2;
						SDLK_F3:	a := KBD_F3;
						SDLK_F4:	a := KBD_F4;
						SDLK_F5:	a := KBD_F5;
						SDLK_F6:	a := KBD_F6;
						SDLK_F7:	a := KBD_F7;
						SDLK_F8:	a := KBD_F8;
						SDLK_F9:	a := KBD_F9;
						SDLK_F10:	a := KBD_F10;
						SDLK_F11:	a := KBD_F11;
						SDLK_F12:	a := KBD_F12;
						SDLK_F13:	a := KBD_F13;
						SDLK_F14:	a := KBD_F14;
						SDLK_F15:	a := KBD_F15;
					else
						if ( 0 < event.key.keysym.unicode ) and ( event.key.keysym.unicode < $80 ) then begin
							a := Char( event.key.keysym.unicode );
						end;
					end;
				end;
  {$ELSE}
				case event.key.keysym.sym of
				SDLK_F1:	begin
							pmsg := QuickPCopy( Config_Directory + 'Demo.bmp' );
							SDL_SaveBmp( Game_Screen , pmsg );
							StrDispose( pmsg );
						end;
    {$IFDEF PATCH_JPSSDL}
					SDLK_F9:		if SkipAnim then SkipAnim := False else SkipAnim := True;
    {$ENDIF PATCH_JPSSDL}
    {$IFDEF PATCH_CHEAT}
					SDLK_F12:		if Cheat_Display then if Cheat_Display_SW then Cheat_Display_SW := False else Cheat_Display_SW := True;
    {$ENDIF PATCH_CHEAT}
					SDLK_Up,SDLK_KP8:	a := RPK_Up;
					SDLK_Down,SDLK_KP2:	a := RPK_Down;
					SDLK_Left,SDLK_KP4:	a := RPK_Left;
					SDLK_Right,SDLK_KP6:	a := RPK_Right;
					SDLK_KP7:		a := RPK_UpLeft;
					SDLK_KP9:		a := RPK_UpRight;
					SDLK_KP1:		a := RPK_DownLeft;
					SDLK_KP3:		a := RPK_DownRight;
					SDLK_Backspace:		a := #8;
					SDLK_KP_Enter:		a := #10;
					SDLK_KP5:		a := '5';
				else
    {$IFDEF PATCH_I18N}
					if( event.key.keysym.unicode <  $80 ) and ( event.key.keysym.unicode > 0 ) then begin
						a := Char( event.key.keysym.unicode );
					end else if (True = GetStringMode) and (event.key.keysym.unicode > 0) then begin
						Unicode := event.key.keysym.unicode;
						a := #$80;
					end;
    {$ELSE PATCH_I18N}
					if( event.key.keysym.unicode <  $80 ) and ( event.key.keysym.unicode > 0 ) then begin
						a := Char( event.key.keysym.unicode );
					end;
    {$ENDIF PATCH_I18N}
				end;
  {$ENDIF}
{$ENDIF SDL2}

{$IFDEF SDL2}
			end else if ( SDL_MOUSEWHEEL = event.type_ ) then begin
				if ( 0 < event.wheel.x ) then begin
					a := KeyMap[ KMC_ButtonWRight ].KCode;
				end else if ( event.wheel.x < 0 ) then begin
					a := KeyMap[ KMC_ButtonWLeft ].KCode;
				end;
				if ( 0 < event.wheel.y ) then begin
					a := KeyMap[ KMC_ButtonWUp ].KCode;
				end else if ( event.wheel.y < 0 ) then begin
					a := KeyMap[ KMC_ButtonWDown ].KCode;
				end;
{$ELSE SDL2}
{$ENDIF SDL2}

			end else if ( event.type_ = SDL_MOUSEButtonDown ) then begin
				{ Return a mousebutton event. }
{$IFDEF PATCH_GH}
				case event.button.button of
				SDL_BUTTON_LEFT:		a := RPK_MouseButton;
				SDL_BUTTON_MIDDLE:		a := KeyMap[ KMC_ButtonMiddle ].KCode;
				SDL_BUTTON_RIGHT:		a := RPK_RightButton;
{$IFDEF SDL2}
{$ELSE SDL2}
				SDL_BUTTON_WHEELUP:		a := KeyMap[ KMC_ButtonWUp ].KCode;
				SDL_BUTTON_WHEELDOWN:		a := KeyMap[ KMC_ButtonWDown ].KCode;
				(SDL_BUTTON_WHEELDOWN+1):	a := KeyMap[ KMC_ButtonWLeft ].KCode; { SDL_BUTTON_X1: (WHEEL-LEFT) }
				(SDL_BUTTON_WHEELDOWN+2):	a := KeyMap[ KMC_ButtonWRight ].KCode; { SDL_BUTTON_X2: (WHEEL-RIGHT) }
{$ENDIF SDL2}
				end;
{$ELSE PATCH_GH}
				if event.button.button = SDL_BUTTON_LEFT then begin
					a := RPK_MouseButton;
				end else if event.button.button = SDL_BUTTON_RIGHT then begin
					a := RPK_RightButton;
				end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
			end else if ( SDL_MOUSEButtonUp = event.type_ ) then begin
				case event.button.button of
				SDL_BUTTON_LEFT:		a := RPK_MouseButtonRelease;
				end;

			end else if ( SDL_MOUSEMOTION = event.type_ ) then begin
				MouseMoved := True;
{$ENDIF PATCH_GH}

{$IFDEF SDL2}
			end else if event.type_ = SDL_WINDOWEVENT then begin
				case event.window.event of
				SDL_WINDOWEVENT_RESIZED: begin
					end;
				end;
{$ELSE SDL2}
			end else if event.type_ = SDL_VIDEORESIZE then begin
{$ENDIF SDL2}

			end;

{$IFDEF SDL2}
  {$IFDEF PATCH_HEART}
		end else if ( not nonblocking ) then begin
  {$ELSE PATCH_HEART}
		end else begin
  {$ENDIF PATCH_HEART}
{$ELSE SDL2}
  {$IFDEF PATCH_HEART}
		end else if nonblocking then begin
			if SDL_GetTicks < ( Last_Clock_Update + KeyWait - SleepGranularity ) then begin
				t := Last_Clock_Update + KeyWait - SDL_GetTicks;
				if 0 < t then begin
					if t > KeysamplingInterval then SDL_Delay( KeysamplingInterval )
					else SDL_Delay( t );
				end;
			end else begin
				a := RPK_TimeEvent;
			end;
  {$ENDIF PATCH_HEART}

		end else begin
{$ENDIF SDL2}

		end;

{$IFDEF PATCH_GH}
		if ('' <> a) then begin
			if RPGKey_LastEvent <> a then begin
				RPGKey_LastEvent := a;
			end else if ( RPK_TimeEvent = a ) then begin
				if SDL_NoTimeEvent then begin
					if MouseMoved then begin
						MouseMoved := False;
						a := RPK_MouseMotion;
					end else begin
						a := '';
					end;
				end else begin
					if MouseMoved then begin
						MouseMoved := False;
						a := RPK_MouseMotion;
					end;
				end;
			end;
		end;
{$ENDIF PATCH_GH}

	{ Keep going until either a character is found, or an error is reported. }
	until ( a <> '' );

{$IFDEF PATCH_GH}
	if not GetStringMode then begin
		i := 1;
		while ( KBD_ERROR <> KCodeAlias[i].KCode_from ) do begin
			if ( KCodeAlias[i].KCode_from = a ) then begin
				a := KCodeAlias[i].KCode_to;
				break;
			end;
			Inc( i );
		end;
	end;
{$ENDIF PATCH_GH}

	if a <> '' then RPGKey_Raw := a[1]
{$IFDEF PATCH_GH}
	else RPGKey_Raw := KBD_NONE;
{$ELSE PATCH_GH}
	else RPGKey_Raw := 'Z';
{$ENDIF PATCH_GH}
end;


{$IFDEF PATCH_GH}
Function GetStringFromUser_Raw( const Prompt: String; const Init_text: String ): String;
var
	dummy: Boolean;
begin
	GetStringFromUser_Raw := GetStringFromUser_Raw( Prompt, Init_text, dummy );
end;

Function GetStringFromUser_Raw( const Prompt: String ): String;
var
	dummy: Boolean;
begin
	GetStringFromUser_Raw := GetStringFromUser_Raw( Prompt, '', dummy );
end;
{$ENDIF PATCH_GH}

{$IFDEF PATCH_GH}
Function GetStringFromUser_Raw( const Prompt: String; const Init_text: String; var canceled: Boolean ): String;
{$ELSE PATCH_GH}
Function GetStringFromUser_Raw( const Prompt: String ): String;
{$ENDIF PATCH_GH}
	{ Does what it says. }
{$IF DEFINED(WINDOWS) and DEFINED(WITHOUT_SDLIM)}
var
	it: String;
	SZ: TSDL_Rect;
	MaxInputWidth: Integer = 0;
	X: Integer;
  {$IFDEF SDL2}
	Parent_X, Parent_Y: Integer;
  {$ELSE SDL2}
  {$ENDIF SDL2}
{$ELSE}
const
  {$IFDEF PATCH_I18N}
	WCLen = 16;
  {$ELSE PATCH_I18N}
	AllowableCharacters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890()-=_+,.?"*';
	MaxInputLength = 80;
  {$ENDIF PATCH_I18N}
var
	A: Char;
	it: String;
	SZ: TSDL_Rect;
  {$IFDEF PATCH_I18N}
    {$IFDEF SDL2}
	Unicode: String;
    {$ELSE SDL2}
	Unicode: Word;
    {$ENDIF SDL2}
	work_dst: Array[0..WCLen] of Char;
	work_pdst: PChar;
	state: ShortInt = 0;
	mbchar_work: String = '';
  {$ENDIF PATCH_I18N}
	MaxInputWidth: Integer = 0;
	X: Integer;
{$ENDIF}
begin
{$IFDEF PATCH_GH}
	canceled := False;
	DrawZoneBorder( ScreenZone[ ZONE_TextInputBigBox , 1 ] - 1 , ScreenZone[ ZONE_TextInputBigBox , 2 ] -1 , ScreenZone[ ZONE_TextInputBigBox , 3 ] + 1 , ScreenZone[ ZONE_TextInputBigBox , 4 ] + 1 , LightCyan );
	ClrZone_Raw( ZONE_TextInputBigBox );
  {$IFDEF PATCH_I18N}
	GameMSG( Prompt , ZONE_TextInputPrompt , InfoGreen );
  {$ELSE PATCH_I18N}
	X := ( ScreenZone[ZONE_TextInputPrompt,3] + ScreenZone[ZONE_TextInputPrompt,1] ) div 2 - ( WidthMBCharStr( Prompt ) div 2 ) + 1;
	if X < GOTOXY_MIN then begin
		X := GOTOXY_MIN;
	end;
	ascii_sdl.GotoXY( X, ScreenZone[ ZONE_TextInputPrompt , 2 ] );
	ascii_sdl.TextColor( InfoGreen );
	ascii_sdl.Write( Prompt );
  {$ENDIF PATCH_I18N}
{$ELSE PATCH_GH}
	DrawZoneBorder( ScreenZone[ ZONE_TextInput , 1 ] - 1 , ScreenZone[ ZONE_TextInput , 2 ] -1 , ScreenZone[ ZONE_TextInput , 3 ] + 1 , ScreenZone[ ZONE_TextInput , 4 ] + 1 , LightCyan );
	ClrZone_Raw( ZONE_TextInput );
  {$IFDEF PATCH_I18N}
	X := ( ScreenZone[ZONE_TextInput,3] + ScreenZone[ZONE_TextInput,1] ) div 2 - ( WidthMBCharStr( Prompt ) div 2 ) + 1;
	if X < GOTOXY_MIN then begin
		X := GOTOXY_MIN;
	end;
	ascii_sdl.GotoXY( X, ScreenZone[ ZONE_TextInput , 4 ] );
	ascii_sdl.TextColor( InfoGreen );
	ascii_sdl.Write( Prompt );
  {$ELSE PATCH_I18N}
	ascii_sdl.GotoXY( ( ScreenZone[ZONE_TextInput,3] + ScreenZone[ZONE_TextInput,1] ) div 2 - ( Length( Prompt ) div 2 ) + 1 , ScreenZone[ ZONE_TextInput , 4 ] );
	ascii_sdl.TextColor( InfoGreen );
	ascii_sdl.Write( Prompt );
  {$ENDIF PATCH_I18N}
{$ENDIF PATCH_GH}
	ascii_sdl.TextColor( InfoHilight );
	ascii_sdl.CursorOn;
	ClipZone( ZONE_TextInput );

	MaxInputWidth := ScreenZone[ ZONE_TextInput , 3 ] - ScreenZone[ ZONE_TextInput , 1 ];
	if 127 < MaxInputWidth then MaxInputWidth := 127;

	{ Initialize string. }
{$IFDEF PATCH_GH}
	it := Init_text;
{$ELSE PATCH_GH}
	it := '';
{$ENDIF PATCH_GH}

{$IF DEFINED(WINDOWS) and DEFINED(WITHOUT_SDLIM)}
	SZ.X := ( ScreenZone[ ZONE_TextInputBigBox , 1 ] * CharWidth );
	SZ.Y := ( ScreenZone[ ZONE_TextInputBigBox , 2 ] * CharHeight );
	SZ.W := ( ( ScreenZone[ ZONE_TextInputBigBox , 3 ] - ScreenZone[ ZONE_TextInputBigBox , 1 ] + 1 ) * CharWidth );
	SZ.H := ( ( ScreenZone[ ZONE_TextInputBigBox , 4 ] - ScreenZone[ ZONE_TextInputBigBox , 2 ] + 1 ) * CharHeight );
  {$IFDEF SDL2}
	SDL_GetWindowPosition( Game_Window , @Parent_X , @Parent_Y );
	it := w32eb.EditBox( Parent_X , Parent_Y , SZ.X , SZ.Y , SZ.W , SZ.H , Prompt , it , canceled );
  {$ELSE SDL2}
	it := w32eb.EditBox( SZ.X , SZ.Y , SZ.W , SZ.H , Prompt , it , canceled );
  {$ENDIF SDL2}
{$ELSE}
	repeat
		ascii_sdl.GotoXY( 1 , 1 );
		ascii_sdl.TextColor( LightGreen );
		ascii_sdl.Write( it );

		SZ.X := (LongInt(VG_X) - 1) * CharWidth;
		SZ.Y := (LongInt(VG_Y) - 1) * CharHeight;
		SZ.W := CharWidth;
		SZ.H := CharHeight;
		SDL_FillRect( Game_Screen , @SZ , SDL_MapRGB( Game_Screen^.Format , 255 , 255 , 255 ) );

		DoFlip;
  {$IFDEF PATCH_I18N}
		A := RPGKey_Raw( Unicode );

    {$IFDEF SDL2}
		if ( 0 < Length(Unicode) ) then begin
    {$ELSE SDL2}
		if (0 < Unicode) then begin
    {$ENDIF SDL2}
			work_pdst := work_dst;
    {$IFDEF SDL2}
			Conv_FromTenc( @(Unicode[1]), Length(Unicode), work_pdst, WCLen );
    {$ELSE SDL2}
			Conv_FromUni16( @Unicode, 2, work_pdst, WCLen );
    {$ENDIF SDL2}
			A := EditMBCharStr( it, 127, MaxInputWidth, #0, work_pdst, state, mbchar_work );
    {$IFDEF PATCH_GH}
		end else if ( RPK_TimeEvent <> A ) and ( RPK_MouseMotion <> A ) then begin
    {$ELSE PATCH_GH}
		end else if RPK_TimeEvent <> A then begin
    {$ENDIF PATCH_GH}
			A := EditMBCharStr( it, 127, MaxInputWidth, A, NIL, state, mbchar_work );
		end;
  {$ELSE PATCH_I18N}
		A := RPGKey_Raw;

		if ( A = #8 ) and ( Length( it ) > 0 ) then begin
			it := Copy( it , 1 , Length( it ) - 1 );
		end else if ( Pos( A , AllowableCharacters ) > 0 ) and ( Length( it ) < MaxInputLength ) then begin
			it := it + A;
		end;
  {$ENDIF PATCH_I18N}
  {$IFDEF PATCH_GH}
	until ( A = #10 ) or ( A = #13 ) or ( A = #27 );
  {$ELSE PATCH_GH}
	until ( A = #13 ) or ( A = #27 );
  {$ENDIF PATCH_GH}
  {$IFDEF PATCH_GH}
	if (#27 = A) then begin
		it := '';
		canceled := True;
	end;
  {$ENDIF PATCH_GH}
{$ENDIF}

	GetStringFromUser_Raw := it;
end;



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: ascii_sdl.pp');
{$ENDIF DEBUG}
	ClrScreen_Raw;
{$IF DEFINED(PATCH_GH) and DEFINED(SDL2) and DEFINED(BSD)}
	fedisableexcept( FE_INVALID );
{$ENDIF}
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: ascii_sdl.pp(finalization)');
{$ENDIF DEBUG}
end;

end.
{$ENDIF ASCII_SDL}
