{$IFDEF ASCII_SDL}
unit ascii_sdlinit;

interface

uses
	sysutils,
{$IFDEF SDL2}
	SDL2,SDL2_TTF
{$ELSE SDL2}
	SDL,SDL_TTF
{$ENDIF SDL2}
{$IFDEF PATCH_AUDIO}
  {$IFDEF SDL2}
	,SDL2_Mixer
  {$ELSE SDL2}
	,SDL_Mixer
  {$ENDIF SDL2}
{$ENDIF PATCH_AUDIO}
	;

{$IFDEF PATCH_AUDIO}
type
	SensibleSoundEffectListPtr = ^SensibleSoundEffectList;
	SensibleSoundEffectList = Record
		Name: String;
		Chunk: PMix_Chunk;
		Next: SensibleSoundEffectListPtr;
	end;
{$ENDIF PATCH_AUDIO}

const
	Game_FontSize: Integer = 12;
	Game_Font: PTTF_Font = NIL;
	ASCII_Width_Of_One_Character: String = 'M';
	I18N_Width_Of_One_Character: String = 'M';
	CharWidth:  LongInt = 8;
	CharHeight: LongInt = 8;
	ScreenWidth:  Integer = 80;
	ScreenHeight: Integer = 25;
	KEY_REPEAT_DELAY = 200;
	KEY_REPEAT_INTERVAL = 75;

var
{$IFDEF SDL2}
	Game_Window: PSDL_Window;
	Game_WindowID: Integer;
{$ELSE SDL2}
{$ENDIF SDL2}
{$IF DEFINED(PATCH_HEART) and DEFINED(SDL2)}
	QRLViewer_WindowID: Integer;
{$ENDIF}
	Game_Screen: PSDL_Surface;
{$IFDEF PATCH_AUDIO}
	SoundEffectList: SensibleSoundEffectListPtr;
{$ENDIF PATCH_AUDIO}



implementation

uses
{$IF DEFINED(SDL2) and (DEFINED(LINUX))}
	math,
{$ENDIF}
{$IFDEF PATCH_GH}
	errmsg,pseudosmartpointer,
{$ELSE PATCH_GH}
  {$IFDEF DEBUG}
	errmsg,
  {$ENDIF DEBUG}
{$ENDIF PATCH_GH}
{$IFDEF PATCH_I18N}
	i18nmsg,
{$ENDIF PATCH_I18N}
	gears_base,
	gears,
	texutil,
	ui4gh;

{$IFDEF SDL2}
const
	WindowName: PChar = 'GearHead Arena ASCII-SDL Version';
{$ELSE SDL2}
{$ENDIF SDL2}


{$IFDEF PATCH_AUDIO}
Procedure Load_SoundEffect;
var
	SoundEffectFileList: SAttPtr;
	P: SAttPtr;
	S: SensibleSoundEffectListPtr;
	PText: PChar;
begin
	SoundEffectFileList := CreateFileList( Sound_Directory + '*.*' );
	SoundEffectList := NIL;

	P := SoundEffectFileList;
	while ( NIL <> P ) do begin
		if ( ( 3 <= Length( P^.Info ) ) and ( '.txt' <> Copy( P^.Info , (Length(P^.Info) - 3) , 4 ) ) and ( '.bak' <> Copy( P^.Info , (Length(P^.Info) - 3) , 4 ) ) ) then begin
			New( S );
  {$IFDEF DEBUG}
			Trace_MemoryLeak('Load_SoundEffect() New',S);
  {$ENDIF DEBUG}
			if ( NIL = S ) then exit;

			S^.Name := P^.Info;
			PText := QuickPCopy( Sound_Directory + P^.Info );
			S^.Chunk := Mix_LoadWAV( PText );
			StrDispose( PText );
			S^.Next := SoundEffectList;

			if ( NIL <> S^.Chunk ) then begin
				SoundEffectList := S;
			end else begin
				ErrorMessage( 'SDL_mixer error at Mix_LoadWAV(' + P^.Info + ') : ' + SDL_GetError );
  {$IFDEF DEBUG}
				Trace_MemoryLeak('Load_SoundEffect() Dispose',S);
				CheckAndNIL_Pointer('Load_SoundEffect() Dispose',S,True);
  {$ENDIF DEBUG}
  {$IFDEF PATCH_GH_PARANOID_SAFER}
				S^.Name[1] := '@';
				S^.Chunk   := NIL;
				S^.Next    := SensibleSoundEffectListPtr(-1);
  {$ENDIF PATCH_GH_PARANOID_SAFER}
				Dispose( S );
			end;
		end;
		P := P^.Next;
	end;

	DisposeSAtt( SoundEffectFileList );
end;

Procedure Dispose_SoundEffect;
var
	S,S2: SensibleSoundEffectListPtr;
begin
	S := SoundEffectList;
	while ( NIL <> S ) do begin
		S2 := S^.Next;

		Mix_FreeChunk( S^.Chunk );
  {$IFDEF DEBUG}
		Trace_MemoryLeak('Dispose_SoundEffect() Dispose',S);
		CheckAndNIL_Pointer('Dispose_SoundEffect() Dispose',S,True);
  {$ENDIF DEBUG}
  {$IFDEF PATCH_GH_PARANOID_SAFER}
		S^.Name[1] := '@';
		S^.Chunk   := NIL;
		S^.Next    := SensibleSoundEffectListPtr(-1);
  {$ENDIF PATCH_GH_PARANOID_SAFER}
		Dispose( S );

		S := S2;
	end;
end;
{$ENDIF PATCH_AUDIO}

Function SearchAndOpenFont( const FontInfo: PFontSearchNameDesc; const arg_ptsize: integer ): PTTF_Font;
const
	TmpLen = 255;
var
	i, j: Integer;
	FontFile: String;
	tmp: array[0..TmpLen] of Char;
	ptsize: Integer;
begin
	SearchAndOpenFont := NIL;
	for j := 0 to (MaxFontSearchNameNum-1) do begin
		for i := 1 to MaxFontSearchDirNum do begin
			if 0 < Length(FontSearchDir[i]) then begin
				FontFile := FontSearchDir[i] + OS_Dir_Separator + FontInfo[j].FontFile;
			end else begin
				FontFile := FontInfo[j].FontFile;
			end;
			ptsize := arg_ptsize;
			if ptsize <= 0 then begin
				ptsize := FontInfo[j].FontSize;
			end;
			if 0 < Length(FontInfo[j].FontFile) then begin
				StrPCopy( tmp, FontFile );
{$IFDEF DEBUG}
				ErrorMessage_fork( '[Name:' + BStr(j) + ';Dir:' + BStr(i) + ']=' + FontFile );
{$ENDIF DEBUG}
				SearchAndOpenFont := TTF_OpenFontIndex( @tmp, ptsize, FontInfo[j].FontFace );
				if (NIL <> SearchAndOpenFont) then
					break;
			end;
		end;
		if (NIL <> SearchAndOpenFont) then
			break;
	end;
end;


Procedure InitVideoEmu;
var
	pmsg: PChar;	{ Gotta convert to pchar, pain in the ass... }
	W: LongInt;
{$IF DEFINED(SDL2) and (DEFINED(LINUX))}
	FPUExceptionMask: TFPUExceptionMask;
{$ENDIF}
begin
	ASCII_Width_Of_One_Character := I18N_Settings('SDLGFX_ASCII_WIDTH_OF_ONE_CHARACTER','M');
	I18N_Width_Of_One_Character := I18N_Settings('SDLGFX_I18N_WIDTH_OF_ONE_CHARACTER','MM');
	ScreenWidth  := ScreenColumns;
	ScreenHeight := ScreenRows;
	Game_FontSize := FontSize_Big;

	SDL_Init( SDL_INIT_VIDEO or SDL_INIT_TIMER or SDL_INIT_AUDIO );
	SDL_SetHint( SDL_HINT_RENDER_SCALE_QUALITY , 'best' );

{$IFDEF SDL2}
{$ELSE SDL2}
        SDL_EnableUNICODE( 1 );
	SDL_EnableKeyRepeat( KEY_REPEAT_DELAY , KEY_REPEAT_INTERVAL );
{$ENDIF SDL2}

	TTF_Init;

	Game_Font := SearchAndOpenFont( @FontSearchName_Big , Game_FontSize );

{$IFDEF PATCH_AUDIO}
	Mix_OpenAudio( MIX_DEFAULT_FREQUENCY , MIX_DEFAULT_FORMAT , 2 , 4096 );
	Mix_AllocateChannels( 16 );
	Load_SoundEffect;
{$ENDIF PATCH_AUDIO}

	pmsg := QuickPCopy( ASCII_Width_Of_One_Character );
{$IFDEF SDL2}
	TTF_SizeUTF8( Game_Font , pmsg , @CharWidth , @CharHeight );
{$ELSE SDL2}
	TTF_SizeUTF8( Game_Font , pmsg , CharWidth , CharHeight );
{$ENDIF SDL2}
	StrDispose( pmsg );
	CharWidth := ( CharWidth div WidthMBCharStr( ASCII_Width_Of_One_Character ) );

	pmsg := QuickPCopy( I18N_Width_Of_One_Character );
{$IFDEF SDL2}
	TTF_SizeUTF8( Game_Font , pmsg , @W , @CharHeight );
{$ELSE SDL2}
	TTF_SizeUTF8( Game_Font , pmsg , W , CharHeight );
{$ENDIF SDL2}
	StrDispose( pmsg );
	W := ( W div WidthMBCharStr( I18N_Width_Of_One_Character ) );
	if ( CharWidth < W ) then begin
		CharWidth := W;
	end;

{$IFDEF SDL2}
	Game_Window := SDL_CreateWindow( WindowName, SDL_WINDOWPOS_UNDEFINED,SDL_WINDOWPOS_UNDEFINED, ScreenWidth * CharWidth, ScreenHeight * CharHeight, 0 );
	Game_WindowID := SDL_GetWindowID( Game_Window );
  {$IFDEF PATCH_HEART}
	QRLViewer_WindowID := -1;
  {$ENDIF PATCH_HEART}
  {$IF DEFINED(SDL2) and (DEFINED(LINUX))}
	FPUExceptionMask := GetExceptionMask;
	SetExceptionMask( FPUExceptionMask + [exInvalidOp, exZeroDivide] );
  {$ENDIF}
	Game_Screen := SDL_GetWindowSurface( Game_Window );
  {$IF DEFINED(SDL2) and (DEFINED(LINUX))}
	SetExceptionMask( FPUExceptionMask );
  {$ENDIF}
{$ELSE SDL2}
	Game_Screen := SDL_SetVideoMode( ScreenWidth * CharWidth, ScreenHeight * CharHeight, 0, SDL_DOUBLEBUF );
{$ENDIF SDL2}
end;



initialization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: ascii_sdlinit.pp');
{$ENDIF DEBUG}
	InitVideoEmu;
end;

finalization
begin
{$IFDEF DEBUG}
	ErrorMessage_fork('DEBUG: ascii_sdlinit.pp(finalization)');
{$ENDIF DEBUG}
	TTF_CloseFont( Game_Font );
	TTF_Quit;
	SDL_FreeSurface( Game_Screen );
{$IFDEF PATCH_AUDIO}
	Dispose_SoundEffect;
	Mix_CloseAudio;
{$ENDIF PATCH_AUDIO}
	SDL_Quit;
end;

end.
{$ENDIF ASCII_SDL}
